import OpenSeries.fisika as fisika

# menghitung masa jenis benda
massa_benda = 14
volume_benda = 8
print(
    f"menghitung massa jenis benda dengan massa benda {massa_benda} gram, dan volume benda {volume_benda} cm3"
)
print(f"hasilnya adalah : {fisika.masa_jenis(massa_benda, volume_benda)}\n")



import unittest
from OpenSeries import fisika as fisika
from OpenSeries.util import error as error


class TestKecepatan(unittest.TestCase):
    def test_angka_valid(self):
        hasil = fisika.kecepatan(100.0, 10.0)
        self.assertEqual(hasil, 10.0)

    def test_dibagi_nol(self):
        hasil = fisika.kecepatan(150.0, 0)
        self.assertEqual(hasil, error.error_dibagi_nol())

    def test_invalid_input(self):
        hasil = fisika.kecepatan("12", 5.0)
        self.assertIsInstance(hasil, str)
        self.assertIn(error.error_tipe_data(["int", "float"]), hasil)

    def test_kecepatan_invalid_input2(self):
        hasil = fisika.kecepatan("12", "50")
        self.assertIsInstance(hasil, str)
        self.assertIn(error.error_tipe_data(["int", "float"]), hasil)


class TestPercepatan(unittest.TestCase):
    def test_percepatan_valid(self):
        hasil = fisika.percepatan(20.0, 5.0)
        self.assertEqual(hasil, 4.0)

    def test_percepatan_dibagi_nol(self):
        hasil = fisika.percepatan(30.0, 0)
        self.assertEqual(error.error_dibagi_nol(), hasil)

    def test_percepatan_nilai_tidak_valid(self):
        hasil = fisika.percepatan("20", 30)
        self.assertIsInstance(hasil, str)
        self.assertIn(error.error_tipe_data(["int", "float"]), hasil)


class TestGerakLurusBeraturan(unittest.TestCase):
    def test_valid(self):
        hasil = fisika.gerak_lurus_beraturan(10.0, 2.0, 3.0)
        self.assertAlmostEqual(hasil, 39.0, places=2)

    def test_invalid_int(self):
        hasil = fisika.gerak_lurus_beraturan(8, 1.5, 2)
        self.assertEqual(hasil, error.error_tipe_data(["float"]))


class TestEnergiKinetik(unittest.TestCase):
    def test_valid_input(self):
        hasil = fisika.energi_kinetik(2.0, 5.0)
        self.assertAlmostEqual(hasil, 25.0)

    def test_invalid_input_mix_tipe_data(self):
        hasil = fisika.energi_kinetik("3", 4.5)
        self.assertEqual(hasil, error.error_tipe_data(["int", "float"]))



import OpenSeries.fisika as fisika

# menghitung hukum arus ohm
kuat_arus = 30
hambatan = 3
print("menghitung hukum ohm")
print(f"dengan kuat arus {kuat_arus}")
print(f"hambatan {hambatan}")
print(f"hasilnya adalah: {fisika.hukum_ohm(kuat_arus, hambatan)}\n")



from OpenSeries.util import error as error
from typing import Union


def kecepatan(jarak: Union[float, int], waktu: Union[float, int]) -> Union[float, str]:
    """
    fungsi untuk menghitung kecepatan

    Parameter:
    jarak (float atau int): jarak tempuh
    waktu (float atau int): waktu tempuh (sekon)

    Return:
    float: hasil dari jarak / waktu
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if isinstance(jarak, (float, int)) and isinstance(waktu, (float, int)):
        try:
            return jarak / waktu
        except ZeroDivisionError:
            # error jika hasil pembagian dibagikan dengan 0
            return error.error_dibagi_nol()
    else:
        return error.error_tipe_data(["int", "float"])


def percepatan(
    kecepatan: Union[float, int], waktu: Union[float, int]
) -> Union[float, str]:
    """
    fungsi untuk menghitung percepatan

    parameter:
    kecepatan (float atau int): kecepatan (m/s)
    waktu (float atau int): waktu tempuh (sekon)

    Return:
    float: hasil dari kecepatan / waktu
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if isinstance(kecepatan, (float, int)) and isinstance(waktu, (float, int)):
        try:
            return kecepatan / waktu
        except ZeroDivisionError:
            # error jika hasil pembagian dibagikan dengan 0
            return error.error_dibagi_nol()
    else:
        return error.error_tipe_data(["int", "float"])


def gerak_lurus_beraturan(
    kecepatan_awal: float, a: float, t: float
) -> Union[float, str]:
    """
    fungsi untuk menghitung jarak yang ditempuh oleh benda yang bergerak lurus beraturan

    parameter:
        kecepatan_awal (float): kecepatan awal (m/s)
        a (float): percepatan (m/s**2)
        t (float): waktu (s)
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if (
        isinstance(kecepatan_awal, float)
        and isinstance(a, float)
        and isinstance(t, float)
    ):
        return kecepatan_awal * t + 0.5 * a * t**2
    else:
        return error.error_tipe_data(["float"])


def energi_kinetik(
    massa: Union[float, int], kecepatan: Union[int, float]
) -> Union[int, float, str]:
    """
    menghitung energi kinetik

    parameter:
        massa (float): massa benda
        kecepatan (float atau int): kecepatan benda
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if isinstance(massa, (int, float)) and isinstance(kecepatan, (int, float)):
        return 0.5 * massa * kecepatan**2
    else:
        return error.error_tipe_data(["int", "float"])


def masa_jenis(
    massa: Union[int, float], volume: Union[int, float]
) -> Union[int, float, str]:
    """
    menghitung masa jenis suatu benda

    parameter:
        massa (float atau int): massa benda
        volume (float atau int): volume benda
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if isinstance(massa, (int, float)) and isinstance(volume, (int, float)):
        try:
            return massa / volume
        except ZeroDivisionError:
            # error jika hasil pembagian dibagikan dengan 0
            return error.error_dibagi_nol()
    else:
        return error.error_tipe_data(["int", "float"])


def energi_potensial(
    m: Union[int, float], g: Union[int, float], h: Union[int, float]
) -> Union[float, int, str]:
    """
    menghitung energi potensial dengan rumus Ep = m * g * h

    Parameter:
        m (float atau int): masa benda
        g (float atau int): gravitasi bumi
        h (float atau int): ketinggian suatu benda
    """
    if (
        not isinstance(m, (float, int))
        and not isinstance(g, (float, int))
        and not isinstance(h, (float, int))
    ):
        return error.error_tipe_data(["float", "int"])
    else:
        return m * g * h


def hukum_ohm(i: Union[float, int], r: Union[float, int]) -> Union[float, int, str]:
    """
    menghitung hukum ohm dengan besar arus listrik yang mengalir
    melalui sebuah hantaran akan berbanding lurus dengan tengangan potensial
    yang diterapkan kepadanya dan berbanding balik dengan hambatan

    Parameter:
        i (float atau int): kuat arus
        r (float atau int): hambatan (ditulis omega)
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if not isinstance(i, (float, int)) and not isinstance(r, (float, int)):
        return error.error_tipe_data(["float", "int"])
    else:
        return i * r



def error_tipe_data(error_data: list[str]) -> str:
    """
    fungsi panggilan untuk menampilkan pesan error

    Parameter:
        error_data (list(str)): tipe data yang ingin diisi
    """
    return f"Kamu memasukkan tipe data yang salah, harus {','.join(error_data)}"


def error_format(format_string: str) -> str:
    """
    fungsi untuk menampilkan format error
    sesuai keinginan dari pesan error

    parameter:
        format_string (str): pesan yang akan diisi
    """
    return "Error: " + format_string


def error_dibagi_nol():
    """
    fungsi panggilan untuk menampilkan pesan error
    pesan error berupa `tidak bisa dibagikan dengan 0
    """
    return "Tidak bisa dibagikan dengan 0"



import OpenSeries.matematika as matematika

nilai_A = 5
nilai_S = 10

print("menghitung probabilitas suatu kejadian")
print(f"dengan jumlah hasil yang menguntungkan :{nilai_A}")
print(f"dan dengan ukuran ruang sampel {nilai_S}")
print(
    f"probabilitas dari kejadiannya adalah: {matematika.peluang_kejadian(nilai_A, nilai_S)}\n"
)



import unittest
from OpenSeries import matematika as matematika
from OpenSeries.util import error as error


class TestKonversi(unittest.TestCase):
    def test_radian_ke_derajat(self):
        self.assertAlmostEqual(matematika.radian_ke_derajat(1), 57.2957795131)
        self.assertAlmostEqual(matematika.radian_ke_derajat(0), 0)
        self.assertEqual(
            matematika.radian_ke_derajat("12"), error.error_tipe_data(["float", "int"])
        )

    def test_derajat_ke_radian(self):
        self.assertAlmostEqual(matematika.derajat_ke_radian(180), 3.1415926535)
        self.assertAlmostEqual(matematika.derajat_ke_radian(0), 0)
        self.assertEqual(
            matematika.derajat_ke_radian("15"), error.error_tipe_data(["float", "int"])
        )

    def test_radian_ke_gradian(self):
        self.assertAlmostEqual(matematika.radian_ke_gradian(180), 11459.1559026)
        self.assertEqual(
            matematika.radian_ke_gradian("128"), error.error_tipe_data(["float", "int"])
        )

    def test_gradian_ke_radian(self):
        self.assertAlmostEqual(matematika.gradian_ke_radian(52), 0.8168140899)
        self.assertEqual(
            matematika.gradian_ke_radian("200"), error.error_tipe_data(["float", "int"])
        )


class TestKelilingLingkaran(unittest.TestCase):
    def test_tipe_data_float(self):
        hasil = matematika.keliling_lingkaran(2.5)
        self.assertEqual(hasil, 15.707963267948966)

    def test_tipe_data_int(self):
        hasil = matematika.keliling_lingkaran(3)
        self.assertAlmostEqual(hasil, 18.84955592153876)

    def test_nilai_input_tidak_valid(self):
        hasil = matematika.keliling_lingkaran("45")
        self.assertEqual(hasil, error.error_tipe_data(["float", "int"]))


class TestDiameterLingkaran(unittest.TestCase):
    def test_tipe_data_float(self):
        hasil = matematika.diameter_lingkaran(2.5)
        self.assertAlmostEqual(hasil, 5.0)

    def test_tipe_data_int(self):
        hasil = matematika.diameter_lingkaran(3)
        self.assertAlmostEqual(hasil, 6.0)

    def test_nilai_input_tidak_valid(self):
        hasil = matematika.diameter_lingkaran("800")
        self.assertEqual(hasil, error.error_tipe_data(["float", "int"]))


class TestPersamaanKuadrat(unittest.TestCase):
    def test_nilai_valid(self):
        hasil = matematika.persamaan_kuadrat(1, -3, 2)
        self.assertAlmostEqual(hasil, 2.0)

    def test_nilai_valid_float(self):
        hasil = matematika.persamaan_kuadrat(1.0, -2.0, 1.0)
        self.assertAlmostEqual(hasil, 1.0)

    def test_nilai_input_tidak_valid(self):
        hasil = matematika.persamaan_kuadrat("12", 2, 3)
        self.assertEqual(hasil, error.error_tipe_data(["float", "int"]))

    def test_nilai_complex(self):
        hasil = matematika.persamaan_kuadrat(1, 2, 5)
        self.assertEqual(hasil, error.error_format("Persamaan memiliki solusi complex"))


class TestRataRata(unittest.TestCase):
    def test_nilai_valid(self):
        hasil = matematika.rata_rata([1, 2, 3, 4, 5])
        self.assertAlmostEqual(hasil, 3.0)

    def test_nilai_valid_float(self):
        hasil = matematika.rata_rata([1.5, 2.5, 3.5, 4.5])
        self.assertAlmostEqual(hasil, 3.0)

    def test_list_kosong(self):
        hasil = matematika.rata_rata([])
        self.assertEqual(hasil, error.error_format("List tidak boleh kosong"))


class TestFaktorial(unittest.TestCase):
    def test_faktorial_nilai_nol(self):
        hasil = matematika.faktorial(0)
        self.assertEqual(hasil, 1)

    def test_faktorial_nilai_int(self):
        hasil = matematika.faktorial(5)
        self.assertEqual(hasil, 120)

    def test_faktorial_nilai_float(self):
        hasil = matematika.faktorial(2.5)
        self.assertEqual(hasil, error.error_tipe_data(["int"]))

    def test_faktorial_nilai_negatif(self):
        hasil = matematika.faktorial(-20)
        self.assertEqual(
            hasil, error.error_format("Tidak bisa menggunakan angka negatif")
        )


class TestPermutasi(unittest.TestCase):
    def test_input_valid(self):
        hasil = matematika.permutasi(5, 2)
        self.assertEqual(hasil, 20)

    def test_input_invalid_float(self):
        hasil = matematika.permutasi(5.2, 2.0)
        self.assertEqual(hasil, error.error_tipe_data(["int"]))

    def test_nilai_input_tidak_valid(self):
        hasil = matematika.permutasi("12", "14")
        self.assertEqual(hasil, error.error_tipe_data(["int"]))


class TestKombinasi(unittest.TestCase):
    def test_input_valid(self):
        hasil = matematika.kombinasi(5, 2)
        self.assertEqual(hasil, 10)

    def test_input_invalid_float(self):
        hasil = matematika.kombinasi(5.2, 2.0)
        self.assertEqual(hasil, error.error_tipe_data(["int"]))

    def test_nilai_input_tidak_valid(self):
        hasil = matematika.kombinasi("12", "14")
        self.assertEqual(hasil, error.error_tipe_data(["int"]))


class TestFPB(unittest.TestCase):
    def test_input_valid(self):
        hasil = matematika.fpb(36, 48)
        self.assertEqual(hasil, 12)

    def test_invalid_input(self):
        hasil = matematika.fpb(15.0, 20.0)
        self.assertEqual(hasil, 5.0)

    def test_angka_invalid(self):
        hasil = matematika.fpb(-36, -40)
        self.assertEqual(hasil, error.error_format("Angka tidak boleh negatif"))

    def test_angka_nol(self):
        hasil = matematika.fpb(0, 48)
        self.assertEqual(hasil, 48)


class TestFaktorPrima(unittest.TestCase):
    def test_input_valid(self):
        hasil = matematika.faktor_prima(30)
        self.assertEqual(hasil, [2, 3, 5])

    def test_input_float(self):
        hasil = matematika.faktor_prima(25.0)
        self.assertEqual(hasil, [5, 5])

    def test_faktor_prima_kosong(self):
        hasil = matematika.faktor_prima(1)
        self.assertEqual(hasil, [])

    def test_input_negatif(self):
        hasil = matematika.faktor_prima(-30)
        self.assertEqual(hasil, error.error_format("Angka tidak boleh negatif"))


class TestMatriksTranspose(unittest.TestCase):
    def test_valid_input(self):
        matriks_a = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
        ekspetasi = [[1, 4, 7], [2, 5, 8], [3, 6, 9]]
        hasil = matematika.transpose_matriks(matriks_a)
        self.assertEqual(hasil, ekspetasi)



import OpenSeries.matematika as matematika

nilai_rata = [2, 3, 5, 5, 6, 1, 2, 3]
print(f"menghitung nilai rata-rata dari {nilai_rata}")
print(f"hasilnya adalah: {matematika.rata_rata(nilai_rata)}\n")



import numpy as np
from typing import Union
from OpenSeries.util import error as pesan_error


def entropy(label: list[int], base: int) -> Union[float, int, str]:
    """
    fungsi menghitung entropy dari suatu fitur pada suatu dataset

    parameter:
        label (list (int)): label fitur yang akan di hitung entropynya
    """
    if not isinstance(label, (list)):
        return pesan_error.error_tipe_data(["list"])
    if not label:
        return pesan_error.error_format("label tidak boleh kosong")
    if not all(isinstance(cek_nilai, int) for cek_nilai in label):
        return pesan_error.error_tipe_data(["int"])
    _, count = np.unique(label, return_counts=True)
    probabilitas = count / len(label)
    probabilitas[probabilitas == 0] = 1

    log = np.log(probabilitas) if base is None else np.log(probabilitas) / np.log(base)
    return np.sum(abs(probabilitas * log))


def standar_deviasi(vektor: np.ndarray) -> Union[float, str]:
    """
    fungsi untuk mengukur penyebaran data terhadap nilai rata-ratanya

    parameter:
        vektor (np.array)
    """
    if not isinstance(vektor, np.ndarray):
        return pesan_error.error_tipe_data(["numpy array"])
    if len(vektor) == 0:
        return pesan_error.error_format("vektor tidak boleh kosong")

    mean_value = np.mean(vektor)
    squared_diff = np.square(vektor - mean_value)
    variansi = np.mean(squared_diff)
    std_deviasi = np.sqrt(variansi)
    return std_deviasi



import OpenSeries.matematika as matematika

nilai1 = 24
nilai2 = 40
print(
    f"hitung nilai fpb (faktor persekutuan terbesar) dari nilai {nilai1} dan {nilai2}"
)
print(f"hasilnya adalah : {matematika.fpb(nilai1, nilai2)}\n")



import OpenSeries.matematika as matematika

print("hitung jari jari lingkaran")
hasil_jari_lingkaran = matematika.luas_lingkaran(4.5)
print(f"hasil dari hitung jari jari lingkaran: {hasil_jari_lingkaran}\n")



# bilangan pi adalah nilai konstant dalam matematika yang merupakan perbandingan keliling
# lingkaran dengan diameternya
pi: float = 3.14159265358979323846

# bilangan euler adalah nilai konstant yang dimana nilai kira-kiranya sama dengan 2.71828
# dan dikarakterisasi dalam berbagai cara
bilangan_euler: float = 2.718281828459045235360



import OpenSeries.matematika as matematika

nilai_suku = 5
suku_pertama = 3
selisih_suku = 2
print("menghitung jumlah deret aritmatika")
print(f"dengan nilai suku: {nilai_suku}")
print(f"suku pertama: {suku_pertama}")
print(f"selisih suku: {selisih_suku}")
print(
    f"hasilnya adalah: {matematika.hitung_jumlah_deret(nilai_suku, suku_pertama, selisih_suku)}\n"
)



import OpenSeries.matematika as matematika

nilai_faktorial = 25
print(f"menghitung nilai faktorial dari angka {nilai_faktorial}")
print(f"hasilnya adalah : {matematika.faktorial(nilai_faktorial)}\n")



import OpenSeries.fisika as fisika

# fungsi menghitung glbb
print("hitung glbb (gerak lurus beraturan)")
result_glbb = fisika.gerak_lurus_beraturan(10, 2, 3)
print(f"hasil dari glbb adalah: {result_glbb}\n")



import OpenSeries.fisika as fisika

# menghitung energi potensial
massa_benda_potensial = 12
gravitasi_bumi = 9.78
ketinggian_benda = 400
print("menghitung energi potensial dari suatu benda")
print(f"dengan massa benda adalah: {massa_benda_potensial}")
print(f"gravitasi bumi {gravitasi_bumi}")
print(f"dan ketinggian benda adalah {ketinggian_benda}")
print(
    f"hasilnya adalah: {fisika.energi_potensial(massa_benda_potensial, gravitasi_bumi, ketinggian_benda)}\n"
)



import unittest
import numpy as np
from OpenSeries import statistika as statistika
from OpenSeries.util import error as pesan_error


class TestFungsiEntropy(unittest.TestCase):
    def test_entropy_dengan_none(self):
        label = [1, 1, 2, 2, 3, 3]
        hasil = statistika.entropy(label, base=2)
        self.assertTrue(np.allclose(hasil, 1.584962500721156))

    def test_nilai_kosong(self):
        label = []
        hasil = statistika.entropy(label, base=2)
        self.assertEqual(hasil, pesan_error.error_format("label tidak boleh kosong"))

    def test_nilai_element_dalam_label(self):
        label = [1, 2, "3", 4]
        hasil = statistika.entropy(label, base=2)
        self.assertEqual(hasil, pesan_error.error_tipe_data(["int"]))

    def test_tipe_data_tuple(self):
        label = (1, 2, 3)
        hasil = statistika.entropy(label, base=2)
        self.assertEqual(hasil, pesan_error.error_tipe_data(["list"]))


class TestFungiStandardDeviasi(unittest.TestCase):
    def test_standard_deviasi_dengan_numpy_array(self):
        vektor = np.array([1, 2, 3, 4, 5])
        hasil = statistika.standar_deviasi(vektor)
        self.assertAlmostEqual(hasil, np.std(vektor), places=2)

    def test_standard_deviasi_dengan_list(self):
        vektor = [1, 2, 3, 4, 5]
        hasil = statistika.standar_deviasi(vektor)
        self.assertEqual(hasil, pesan_error.error_tipe_data(["numpy array"]))

    def test_standard_deviasi_dengan_numpy_kosong(self):
        vektor = np.array([])
        hasil = statistika.standar_deviasi(vektor)
        self.assertEqual(hasil, pesan_error.error_format("vektor tidak boleh kosong"))



from OpenSeries.util import constant as constant
from OpenSeries.util import error as pesan_error
from typing import Union, Sequence
import math


def radian_ke_derajat(radian: Union[float, int]) -> Union[float, str]:
    """
    mengubah nilai radian ke derajat

    parameter:
        radian (float atau integer): nilai radian
    """
    if not isinstance(radian, (float, int)):
        return pesan_error.error_tipe_data(["float", "int"])
    else:
        return radian * (180 / constant.pi)


def derajat_ke_radian(derajat: Union[float, int]) -> Union[float, str]:
    """
    mengubah nilai derajat ke nilai radian

    parameter:
        derajat (float atau int): nilai derajat
    """
    if not isinstance(derajat, (float, int)):
        return pesan_error.error_tipe_data(["float", "int"])
    else:
        return derajat * (constant.pi / 180)


def radian_ke_gradian(radian: Union[float, int]) -> Union[float, str]:
    """
    mengubah nilai radian ke gradian

    Parameter:
        radian (float atau int): nilai radian
    """
    if not isinstance(radian, (float, int)):
        return pesan_error.error_tipe_data(["float", "int"])
    else:
        return radian * (200 / constant.pi)


def gradian_ke_radian(gradian: Union[float, int]) -> Union[float, str]:
    """
    mengubah nilai gradian ke radian

    parameter:
        gradian (float atau int): nilai gradian
    """
    if not isinstance(gradian, (float, int)):
        return pesan_error.error_tipe_data(["float", "int"])
    else:
        return gradian * (constant.pi / 200)


def luas_lingkaran(jari: Union[float, int]) -> Union[float, str]:
    """
    menghitung luas lingkaran

    Parameter:
    jari-jari (float atau integer): jari jari yang akan dihitung
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if isinstance(jari, (float, int)):
        return constant.pi * jari**2
    else:
        return pesan_error.error_tipe_data(["float", "int"])


def keliling_lingkaran(jari: Union[float, int]) -> Union[float, str]:
    """
    menghitung keliling lingkaran

    parameter:
        jari (float atau integer): jari-jari lingkaran
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if isinstance(jari, (float, int)):
        return 2 * constant.pi * jari
    else:
        return pesan_error.error_tipe_data(["float", "int"])


def diameter_lingkaran(jari: Union[float, int]) -> Union[float, str]:
    """
    menghitung diameter lingkaran

    parameter:
        jari (float atau integer): jari-jari lingkaran
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if isinstance(jari, (float, int)):
        return 2 * jari
    else:
        return pesan_error.error_tipe_data(["float", "int"])


def persamaan_kuadrat(
    a: Union[int, float], b: Union[int, float], c: Union[int, float]
) -> Union[float, int, str]:
    """
    menghitung persamaan linear

    parameter:
        a (float atau integer): nilai a
        b (float atau integer): nilai b
        c (float atau integer): nilai c

    return:
        (float atau integer): hasil persamaan linear dari 3 bilangan a, b, dan c
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if (
        isinstance(a, (int, float))
        and isinstance(b, (int, float))
        and isinstance(c, (int, float))
    ):
        diskriminan = b**2 - 4 * a * c
        if diskriminan >= 0:
            akar1 = (-b + math.sqrt(diskriminan)) / (2 * a)
            return akar1
        else:
            return pesan_error.error_format("Persamaan memiliki solusi complex")
    else:
        return pesan_error.error_tipe_data(["float", "int"])


def rata_rata(nilai: Sequence[Union[int, float]]) -> Union[int, float, str]:
    """
    menghitung nilai rata-rata

    parameter:
        nilai (list(float atau integer)): nilai yang dihitung
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if isinstance(nilai, list):
        # mengecek apakah nilai dalam list kodong
        if not nilai:
            return pesan_error.error_format("List tidak boleh kosong")
        # membuat looping untuk memecah nilai yang terdapat pada list
        for cek_nilai in nilai:
            # mengecek nilai dalam list apakah semua tipe data berbentuk int
            # atau float, jika tidak error
            if not isinstance(cek_nilai, (int, float)):
                return pesan_error.error_tipe_data(["float", "int"])
        # menghitung nilai rata-rata
        return sum(nilai) / len(nilai)
    else:
        return pesan_error.error_tipe_data(["float", "int"])


def faktorial(nilai: int) -> Union[int, float, str]:
    """
    menghitung produk dari semua bilangan bulat positif
    contoh `4! = 24 = 4 x 3 x 2 x 1`

    parameter:
        nilai (int): nilai yang akan di faktorial
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if isinstance(nilai, int):
        if nilai == 0 or nilai == 1:
            return 1
        elif nilai < 0:
            return pesan_error.error_format("Tidak bisa menggunakan angka negatif")
        else:
            return int(nilai * faktorial(nilai - 1))
    else:
        return pesan_error.error_tipe_data(["int"])


def permutasi(nilai: int, r: int) -> Union[int, float, str]:
    """
    menghitung nilai permutasi dari n objek yang diambil dari r

    parameter:
        nilai (int): nilai objek
        r (int): jumlah objek yang diambil
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if not isinstance(nilai, int) or not isinstance(r, int):
        return pesan_error.error_tipe_data(["int"])
    else:
        faktorial_nilai = faktorial(nilai)
        faktorial_nilai_r = faktorial(nilai - r)

        if isinstance(faktorial_nilai, str) or isinstance(faktorial_nilai_r, str):
            return pesan_error.error_tipe_data(["int"])
        else:
            return faktorial_nilai / faktorial_nilai_r


def kombinasi(nilai: int, r: int) -> Union[int, float, str]:
    """
    menghitung nilai kombasi dari n objek yang diambil dari r

    parameter:
        nilai (int): nilai objek
        r (int): jumlah objek yang diambil
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if isinstance(nilai, (int)) and isinstance(r, int):
        faktorial_nilai = faktorial(nilai)
        faktorial_r = faktorial(r)
        faktorial_nilai_r = faktorial(nilai - r)

        if (
            isinstance(faktorial_nilai, str)
            or isinstance(faktorial_r, str)
            or isinstance(faktorial_nilai_r, str)
        ):
            return pesan_error.error_tipe_data(["int"])
        else:
            return faktorial_nilai / (faktorial_r * faktorial_nilai_r)
    else:
        return pesan_error.error_tipe_data(["int"])


def fpb(
    bilangan_pertama: Union[int, float], bilangan_kedua: Union[int, float]
) -> Union[int, float, str]:
    """
    menghitung faktor persekutuan terbesar dari dua buah bilangan

    parameter:
        bilangan_pertama (float atau integer): bilagan pertama
        bilangan_kedua (float atau integer): bilangan kedua
    """
    # mengecek apakah variable tersebut bertipe data int atau float
    # jika tidak maka error
    if isinstance(bilangan_pertama, (int, float)) and isinstance(
        bilangan_kedua, (int, float)
    ):
        if bilangan_pertama < 0 and bilangan_kedua < 0:
            return pesan_error.error_format("Angka tidak boleh negatif")
        else:
            while bilangan_kedua:
                bilangan_pertama, bilangan_kedua = (
                    bilangan_kedua,
                    bilangan_pertama % bilangan_kedua,
                )
            return abs(bilangan_pertama)
    else:
        return pesan_error.error_tipe_data(["float", "int"])


def faktor_prima(n: int) -> Union[list[int], str]:
    """
    membuat fungsi untuk mengurutkan nilai faktor prima

    parameter:
        n (int): rentang angka yang mau di tampilkan bilangan faktor prima
    """
    # mengecek apakah variable n bertipe data integer atau float
    if not isinstance(n, (int)) and not isinstance(n, (float)):
        # menampilkan pesan error jika tipe data salah
        return pesan_error.error_tipe_data(["float", "int"])
    else:
        if n < 0:
            return pesan_error.error_format("Angka tidak boleh negatif")
        else:
            # jika desimal mengubah bilangan ke integer
            n = int(n)
            i = 2
            faktor = []
            while i * i <= n:
                if n % i:
                    i += 1
                else:
                    n //= i
                    # memasukkan hasil faktor prima ke dalam list
                    faktor.append(i)
            if n > 1:
                faktor.append(n)
            return faktor


def peluang_kejadian(
    kejadian: Union[int, float], ukuran_sampel: Union[int, float]
) -> Union[int, float, str]:
    """
    menghitung probabilitas dari suatu kejadian

    parameter:
        kejadian (float atau integer): jumlah hasil yang menguntungkan (n(A))
        ukuran sampel (float atau integer): ukurang ruang sampel (n(S))
    """
    # mengecek apakah variable kejadian, ukuran sampel bertipe data integer atau float
    if not isinstance(kejadian, (float, int)) and not isinstance(
        ukuran_sampel, (float, int)
    ):
        return pesan_error.error_tipe_data(["float", "int"])
    else:
        try:
            return kejadian / ukuran_sampel
        except ZeroDivisionError:
            # menampilkan pesan error jika dibagi dengan 0
            return pesan_error.error_dibagi_nol()


def hitung_jumlah_deret(
    n: Union[float, int], a: Union[float, int], b: Union[float, int]
) -> Union[float, int, str]:
    """
    menghitung jumlah deret aritmatika

    parameter:
        n (float atau integer): jumlah suku dalam deret
        a (float atau integer): suku pertama dalam deret
        b (float atau integer): selisih antara dua suku berturut-turut
    """
    # mengecek apakah variable n, a dan b bertipe data integer atau float
    if (
        not isinstance(n, (float, int))
        and not isinstance(a, (float, int))
        and not isinstance(b, (float, int))
    ):
        return pesan_error.error_tipe_data(["float", "int"])
    else:
        return 0.5 * n * (2 * a + (n - 1) * b)


def transpose_matriks(
    matriks: list[list[Union[float, int]]],
) -> Union[list[list[Union[float, int]]], str]:
    """
    fungsi untuk transpose matrix

    parameter:
        matriks: list[float, int]: matriks yang akan di transpose
    """
    if not isinstance(matriks, list):
        return pesan_error.error_tipe_data(["list"])
    else:
        return [
            [matriks[j][i] for j in range(len(matriks))] for i in range(len(matriks[0]))
        ]



from setuptools import setup, find_packages
from os import path
import io
import platform

# membuka file README
with open("README.md") as file_readme:
    readme = file_readme.read()

# buat installasi requirement dari requirements agar auto install
info = path.abspath(path.dirname(__file__))
with io.open(path.join(info, "requirements.txt"), encoding="utf-8") as file:
    core_require = file.read().split("\n")
    if platform.system == "windows":
        core_require.append("pywin32")

install_require = [x.strip() for x in core_require if "git+" not in x]

# setup nama project
setup(
    # nama dari project
    name="OpenSeries",
    # versi dari project
    version="1.0.0",
    # deskripsi singkat dari project
    description="library untuk membantu temen-temen SMA/SMK/Sederajat",
    # deskripsi detail tentang project
    long_description=str(readme),
    # url atau sumber dari projek
    url="https://github.com/bellshade/OpenSeries",
    # maintainer, developer yang membuat dari project
    author="bellshade, wpu, kelas terbuka",
    packages=find_packages(),
    # jika ada folder tambahan dari project ditambahkan dalam dictionary
    package_data={"OpenSeries": ["util/*"]},
    # minimum python yang dibutuhkan untuk menginstall dari project ini
    python_requires=">=3.10",
    # mengklasifikasi project dari OpenSeries
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Operating System :: OS Independent",
    ],
    # fungsi untuk menginstall package tambahan dari requirements
    install_requires=install_require,
    # lisensi dari project
    license="MIT License",
    project_urls={
        "Bug Reports": "https://github.com/bellshade/OpenSeries/issues",
        "Source": "https://github.com/bellshade/OpenSeries",
    },
)



import OpenSeries.fisika as fisika

# fungsi menghitung energi kinetik
massa_benda = 14
kecepatan_benda = 23.4
print(
    f"menghitung energi kinetik pada benda {massa_benda} kg dengan kecepatan {kecepatan_benda} m/s"
)
print(
    f"hasilnya adalah : {fisika.energi_kinetik(massa_benda, kecepatan_benda)} joule\n"
)



import unittest
from testing.matematika_test import (
    TestKonversi,
    TestKelilingLingkaran,
    TestDiameterLingkaran,
    TestPersamaanKuadrat,
    TestRataRata,
    TestFaktorial,
    TestPermutasi,
    TestKombinasi,
    TestFPB,
    TestFaktorPrima,
    TestMatriksTranspose,
)

from testing.fisika_test import (
    TestKecepatan,
    TestPercepatan,
    TestGerakLurusBeraturan,
    TestEnergiKinetik,
)

from testing.statistika_test import TestFungsiEntropy, TestFungiStandardDeviasi

if __name__ == "__main__":
    testing_matematika: list = [
        TestKelilingLingkaran,
        TestKonversi,
        TestDiameterLingkaran,
        TestPersamaanKuadrat,
        TestRataRata,
        TestFaktorial,
        TestPermutasi,
        TestKombinasi,
        TestFPB,
        TestFaktorPrima,
        TestMatriksTranspose,
    ]

    testing_fisika: list = [
        TestKecepatan,
        TestPercepatan,
        TestGerakLurusBeraturan,
        TestEnergiKinetik,
    ]

    testing_statistika: list = [
        TestFungsiEntropy,
        TestFungiStandardDeviasi,
    ]

    all_tests = unittest.TestSuite()

    for testing_math in testing_matematika:
        all_tests.addTest(unittest.TestLoader().loadTestsFromTestCase(testing_math))

    for testing_physic in testing_fisika:
        all_tests.addTest(unittest.TestLoader().loadTestsFromTestCase(testing_physic))

    for testing_statistik in testing_statistika:
        all_tests.addTest(
            unittest.TestLoader().loadTestsFromTestCase(testing_statistik)
        )

    unittest.TextTestRunner(verbosity=2).run(all_tests)
